@@ -45,13 +45,12 @@
 #include <locale.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
-#include <libgnomeui/libgnomeui.h>
 #include <iostream>
 
 using namespace OpenBabel;
 using namespace std;
 
-gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone): Object(DocumentType)
+gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone): Document()
 {
 	m_pApp = App;
 	m_pView = NULL;
@@ -457,22 +456,9 @@ void gcpDocument::Save()
 	{
 		ExportOB();
 	}
-	else{
-puts(m_filename);
-	struct stat buf;
-	gint err;
-		err = stat(m_filename, &buf);
-		gint result = GTK_RESPONSE_YES;
-		if (!err)
-		{
-puts("exists");
-			gchar * message = g_strdup_printf(_("File %s\nexists, overwrite?"), m_filename);
-			GtkDialog* Box = GTK_DIALOG(gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
-			result = gtk_dialog_run(Box);
-			gtk_widget_destroy(GTK_WIDGET(Box));
-			g_free(message);
-		}
-	BuildXMLTree();}
+	else
+		BuildXMLTree();
+	m_bDirty = false;
 }
 
 bool gcpDocument::Load(xmlNodePtr root)
@@ -484,7 +470,6 @@ bool gcpDocument::Load(xmlNodePtr root)
 	g_date_clear(&CreationDate, 1);
 	g_date_clear(&RevisionDate, 1);
 	m_FileType = UNDEFINED;
-	//read title first, then atoms (isolated atoms could be present), then molecules, and finally reactions	//FIXME reactions not available
 	xmlNodePtr node, child;
 	char* tmp;
 	Object* pObject;
@@ -525,40 +510,19 @@ bool gcpDocument::Load(xmlNodePtr root)
 	}
 
 	m_bIsLoading = true;
-	node = GetNodeByName(root, "molecule");
-	while (node)
-	{
-		pObject = new gcpMolecule();
-		AddChild(pObject);
-		if (!pObject->Load(node)) {m_bIsLoading = false; return false;}
-		m_pView->Update(pObject);
-		node = GetNextNodeByName(node->next, "molecule");
-	}
-
-	string str;
-	node = GetNodeByName(root, "object");//FIXME: should load all children with only one loop
-	while (node)
-	{
-		child = node->children;
-		str = (const char*)child->name;
-		Object* pObject = Object::CreateObject(str, this);
-		if (pObject)
-		{
-			pObject->Load(child); 
-			AddObject(pObject);
+	node = root->children;
+	while (node) {
+		child = (strcmp ((const char*) node->name, "object"))? node: node->children;
+		pObject = CreateObject ((const char*) child->name, this);
+		if (pObject) {
+			if (!pObject->Load (child))
+				delete pObject;
+			else
+				m_pView->AddObject (pObject);
 		}
-		else cerr << "Warning: unknown object: " << str << endl;
-		node = GetNextNodeByName(node->next, "object");
-	}
-	node = GetNodeByName(root, "text");//FIXME: should load all children with only one loop
-	while (node)
-	{
-		gcpText* text = new gcpText();
-		if (text->Load(node))
-			AddObject(text);
-		else delete text;
-		node = GetNextNodeByName(node->next, "text");
+		node = node->next;
 	}
+	m_pView->Update (this);
 	m_bIsLoading = false;
 	return true;
 }
@@ -572,6 +536,7 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 {
 	xmlDocPtr xml;
 	xmlNodePtr node;
+	xmlNsPtr ns;
 	char *old_num_locale, *old_time_locale;
 
 	xml = xmlNewDoc((xmlChar*)"1.0");
@@ -584,6 +549,8 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 	setlocale(LC_TIME, "C");
 	
 	xml->children =  xmlNewDocNode(xml, NULL, (xmlChar*)"chemistry", NULL);
+	ns = xmlNewNs (xml->children, (xmlChar*) "http://www.nongnu.org/gchempaint", (xmlChar*) "gcp");
+	xmlSetNs (xml->children, ns);
 	SaveId(node);
 	if (!g_date_valid(&CreationDate))
 		g_date_set_time(&CreationDate, time(NULL));
@@ -759,19 +726,31 @@ void gcpDocument::Remove(Object* pObject)
 		case BondType:
 			RemoveBond((gcpBond*)pObject);
 			break;
-		case MoleculeType:
-			((gcpMolecule*)pObject)->Clear();
-		default:
-			m_pView->Remove(pObject);
-			map<string, Object*>::iterator i;
-			Object* object = pObject->GetFirstChild(i);
-			while (object)
-			{
-				m_pView->Remove(object);
-				delete object;
-				object = pObject->GetNextChild(i);
+		case MoleculeType: {
+				((gcpMolecule*)pObject)->Clear();
+				m_pView->Remove(pObject);
+				map<string, Object*>::iterator i;
+				Object* object = pObject->GetFirstChild(i);
+				while (object)
+				{
+					m_pView->Remove(object);
+					delete object;
+					object = pObject->GetNextChild(i);
+				}
+				delete pObject;
+			}
+			break;
+		default: {
+				m_pView->Remove(pObject);
+				map<string, Object*>::iterator i;
+				Object* object = pObject->GetFirstChild(i);
+				while (object)
+				{
+					Remove(object);
+					object = pObject->GetFirstChild(i);
+				}
+				delete pObject;
 			}
-			delete pObject;
 			break;
 	}
 }
@@ -870,6 +849,7 @@ void gcpDocument::FinishOperation()
 	}
 	m_pApp->ActivateMenu("Redo", false);
 	m_pCurOp = NULL;
+	m_bDirty = true;
 }
 
 void gcpDocument::AbortOperation()
