@@ -35,8 +35,8 @@
 #include "settings.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
-#include "libgcpcanvas/gcp-canvas-line.h"
-#include <chemistry/element.h>
+#include "libgcpcanvas/gcp-canvas-bpath.h"
+#include <gcu/element.h>
 #include <math.h>
 
 extern GtkTextTagTable *TextTagTable;
@@ -335,8 +335,7 @@ void gcpFragment::Add(GtkWidget* w)
 	gnome_canvas_rich_text_ext_set_buffer(GNOME_CANVAS_RICH_TEXT_EXT(item), m_buf);
 	g_object_set_data(G_OBJECT(item), "object", this);
 	g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
-	char charge = m_Atom->GetCharge();
-	if (charge) AddChargeItem(group);
+	AddChargeItem(group);
 	pData->Items[this] = group;
 	if (g_object_get_data(G_OBJECT(item), "realized")) OnChanged(m_buf);
 }
@@ -348,81 +347,107 @@ void gcpFragment::AddChargeItem(GnomeCanvasGroup* group)
 	GtkWidget* pWidget = pDoc->GetView()->GetWidget();
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(pWidget), "data");
 	double x, y;
-	int align = GetChargePosition(m_Atom, x, y);
-	x *= pData->ZoomFactor;
-	y *= pData->ZoomFactor;
-	switch (align)
-	{
-		case -2:
-			y += 6.0;//FIXME: change 6.0 end other constants to variables
-			break;
-		case -1:
-			x -= 6.0;
-			break;
-		case 1:
-			x += 6.0;
-			break;
-		case 2:
-			y -= 6.0;
-			break;
-	}
-	GnomeCanvasGroup* chargegroup = GNOME_CANVAS_GROUP(gnome_canvas_item_new(group, gnome_canvas_group_ext_get_type(), NULL));
-	g_object_set_data ((GObject*)group, "charge", chargegroup);
-	gnome_canvas_item_new(
-					chargegroup,
-					gnome_canvas_ellipse_ext_get_type(),
-					"width_units", 1.0,
-					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
-					"x1", x - 4.0,
-					"x2", x + 4.0,
-					"y1", y - 4.0,
-					"y2", y + 4.0,
-					NULL);
-	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
-	points->coords[0] = x - 2.5;
-	points->coords[1] = points->coords[3] = y;
-	points->coords[2] = x + 2.5;
-	gnome_canvas_item_new(
-					chargegroup,
-					gnome_canvas_line_ext_get_type(),
-					"points", points,
-					"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-					"width_units", 1.0,
-					NULL);
-	if (m_Atom->GetCharge() > 0)
-	{	
-		points->coords[0] = points->coords[2] = x;
-		points->coords[1] = y - 2.5;
-		points->coords[3] = y + 2.5;
-		gnome_canvas_item_new(
-						chargegroup,
-						gnome_canvas_line_ext_get_type(),
-						"points", points,
-						"fill_color", (pData->IsSelected(this))? SelectColor: Color,
+	GnomeCanvasItem* item;
+	GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
+	if (!m_Atom->GetCharge()) {
+		x = m_x * pData->ZoomFactor;
+		y = m_y * pData->ZoomFactor;
+		item = gnome_canvas_item_new(
+						group,
+						gnome_canvas_ellipse_ext_get_type(),
+						"width_units", 0.,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"x1", x - 1.,
+						"x2", x + 1.,
+						"y1",  y - 1.,
+						"y2", y + 1.,
+						NULL);
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+		g_object_set_data ((GObject*)group, "circle", item);
+		gnome_canvas_path_def_moveto (path, x - 0.01, y);
+		gnome_canvas_path_def_lineto (path, x + 0.01, y);
+		item = gnome_canvas_item_new(
+						group,
+						gnome_canvas_bpath_ext_get_type(),
+						"bpath", path,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"width_units", 0.01,
+						NULL);
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+		g_object_set_data ((GObject*)group, "charge", item);
+	} else {
+		int align = GetChargePosition(m_Atom, x, y);
+		x *= pData->ZoomFactor;
+		y *= pData->ZoomFactor;
+		switch (align)
+		{
+			case -2:
+				y += 6.0;//FIXME: change 6.0 end other constants to variables
+				break;
+			case -1:
+				x -= 6.0;
+				break;
+			case 1:
+				x += 6.0;
+				break;
+			case 2:
+				y -= 6.0;
+				break;
+		}
+		item = gnome_canvas_item_new(
+						group,
+						gnome_canvas_ellipse_ext_get_type(),
+						"width_units", 1.0,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"x1", x - 4.0,
+						"x2", x + 4.0,
+						"y1", y - 4.0,
+						"y2", y + 4.0,
+						NULL);
+		g_object_set_data ((GObject*)group, "circle", item);
+		gnome_canvas_path_def_moveto (path, x - 2.5, y);
+		gnome_canvas_path_def_lineto (path, x + 2.5, y);
+		if (m_Atom->GetCharge() > 0)
+		{	
+			gnome_canvas_path_def_moveto (path, x, y - 2.5);
+			gnome_canvas_path_def_lineto (path, x, y + 2.5);
+		}
+		item = gnome_canvas_item_new(
+						group,
+						gnome_canvas_bpath_ext_get_type(),
+						"bpath", path,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
 						"width_units", 1.0,
 						NULL);
+		g_object_set_data ((GObject*)group, "charge", item);
 	}
+	gnome_canvas_path_def_unref (path);
 }
 
 void gcpFragment::SetSelected(GtkWidget* w, int state)
 {
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
-	gchar* color;
+	gchar *chargecolor, *color;
 	switch (state)
 	{	
-		case SelStateUnselected: color = "white"; break;
-		case SelStateSelected: color = SelectColor; break;
-		case SelStateUpdating: color = AddColor; break;
-		case SelStateErasing: color = DeleteColor; break;
+		case SelStateUnselected: color = "white"; chargecolor = "black"; break;
+		case SelStateSelected: chargecolor = color = SelectColor; break;
+		case SelStateUpdating: chargecolor = color = AddColor; break;
+		case SelStateErasing: chargecolor = color = DeleteColor; break;
 	}
-	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")), "fill_color", color, NULL);
+	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
+				"fill_color", color, NULL);
+	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "circle")),
+				"outline_color", chargecolor, NULL);
+	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "charge")),
+				"outline_color", chargecolor, NULL);
 }
 
 void gcpFragment::Update(GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
-	GnomeCanvasGroup *chargegroup, *group = pData->Items[this];
+	GnomeCanvasGroup *group = pData->Items[this];
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "fragment")),
 						"x", m_x * pData->ZoomFactor - m_lbearing,
 						"y", m_y * pData->ZoomFactor - m_ascent + m_CHeight,
@@ -435,10 +460,72 @@ void gcpFragment::Update(GtkWidget* w)
 						"x2", m_x * pData->ZoomFactor + m_length + pData->Padding - m_lbearing,
 						"y2", m_y * pData->ZoomFactor + m_height + pData->Padding - m_ascent + m_CHeight,
 						NULL);
-	chargegroup = (GnomeCanvasGroup*) g_object_get_data(G_OBJECT(group), "charge");
-	if (chargegroup) gtk_object_destroy(GTK_OBJECT(chargegroup));
-	char charge = m_Atom->GetCharge();
-	if (charge) AddChargeItem(group);
+	GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
+	void* item;
+	double x, y;
+	if (!m_Atom->GetCharge()) {
+		x = m_x * pData->ZoomFactor;
+		y = m_y * pData->ZoomFactor;
+		item = g_object_get_data(G_OBJECT(group), "circle");
+		g_object_set (item,
+						"width_units", 0.,
+						"x1", x - 1,
+						"x2", x + 1,
+						"y1", y - 1,
+						"y2", y + 1,
+						NULL);
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+		gnome_canvas_path_def_moveto (path, x - 0.01, y);
+		gnome_canvas_path_def_lineto (path, x + 0.01, y);
+		item = g_object_get_data(G_OBJECT(group), "charge");
+		g_object_set (item,
+						"bpath", path,
+						"width_units", 0.,
+						NULL);
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+	} else {
+		int align = GetChargePosition(m_Atom, x, y);
+		x *= pData->ZoomFactor;
+		y *= pData->ZoomFactor;
+		switch (align)
+		{
+			case -2:
+				y += 6.0;//FIXME: change 6.0 end other constants to variables
+				break;
+			case -1:
+				x -= 6.0;
+				break;
+			case 1:
+				x += 6.0;
+				break;
+			case 2:
+				y -= 6.0;
+				break;
+		}
+		item = g_object_get_data(G_OBJECT(group), "circle");
+		g_object_set (item,
+						"width_units", 1.0,
+						"x1", x - 4.0,
+						"x2", x + 4.0,
+						"y1", y - 4.0,
+						"y2", y + 4.0,
+						NULL);
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
+		gnome_canvas_path_def_moveto (path, x - 2.5, y);
+		gnome_canvas_path_def_lineto (path, x + 2.5, y);
+		if (m_Atom->GetCharge() > 0)
+		{	
+			gnome_canvas_path_def_moveto (path, x, y - 2.5);
+			gnome_canvas_path_def_lineto (path, x, y + 2.5);
+		}
+		item = g_object_get_data(G_OBJECT(group), "charge");
+		g_object_set (item,
+						"bpath", path,
+						"width_units", 1.0,
+						NULL);
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
+	}
+	gnome_canvas_path_def_unref (path);
 }
 
 xmlNodePtr gcpFragment::Save(xmlDocPtr xml)
@@ -1066,3 +1153,18 @@ bool gcpFragment::Validate()
 	}
 	return true;
 }
+
+void gcpFragment::Transform2D(Matrix2D& m, double x, double y)
+{
+	m_x -= x;
+	m_y -= y;
+	m.Transform (m_x, m_y);
+	m_x += x;
+	m_y += y;
+	m_Atom->SetCoords(m_x, m_y);
+}
+	
+double gcpFragment::GetYAlign ()
+{
+	return m_y;
+}
