@@ -36,6 +36,7 @@
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include <math.h>
 #include <pango/pango-context.h>
+#include <gdk/gdkkeysyms.h>
 
 /*
 Derivation of a new widget from gnome_canvas with an event for updating canvas size
@@ -198,6 +199,8 @@ gcpView::gcpView(gcpDocument *pDoc, bool Embedded)
 	m_DefaultArrowHeadB = DefaultArrowHeadB;
 	m_DefaultArrowHeadC = DefaultArrowHeadC;
 	m_DefaultArrowDist = DefaultArrowDist;
+	m_DefaultArrowPadding = DefaultArrowPadding;
+	m_DefaultSignPadding = DefaultSignPadding;
 	m_width = 400;
 	m_height = 300;
 	m_ActiveRichText = NULL;
@@ -206,12 +209,6 @@ gcpView::gcpView(gcpDocument *pDoc, bool Embedded)
 
 gcpView::~gcpView()
 {
-//	if (!bCloseAll && !m_bEmbedded)
-	if (!m_bEmbedded)
-	{
-		void* Item = g_object_get_data(G_OBJECT(m_pWidget), "menu");
-		if (GTK_IS_WIDGET(Item)) gtk_widget_destroy(GTK_WIDGET(Item));
-	}
 	g_object_unref(G_OBJECT(m_PangoContext));
 	if (m_sFontName) g_free(m_sFontName);
 	if (m_sSmallFontName) g_free(m_sSmallFontName);
@@ -285,7 +282,10 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 				return true;
 			}
 			case 3:
-				if (Obj) Obj->ShowContextualMenu(event->button.button, event->button.time);
+				if (pActiveTool->OnRightButtonClicked(this, Obj, event->button.x, event->button.y))
+					return true;
+				if (Obj)
+					Obj->ShowContextualMenu(event->button.button, event->button.time);
 		}
 		break;
 	case GDK_MOTION_NOTIFY:
@@ -293,7 +293,7 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 		pActiveTool->OnDrag(x, y, event->button.state);
 		return true;
 	case GDK_BUTTON_RELEASE:
-		switch (event->button.button)
+	switch (event->button.button)
 		{
 		case 1:
 			if (!bDragging) break;
@@ -344,6 +344,8 @@ GtkWidget* gcpView::CreateNewWidget()
 		m_pData->HashWidth = m_DefaultHashWidth * DefaultZoomFactor;
 		m_pData->HashDist = m_DefaultHashDist * DefaultZoomFactor;
 		m_pData->StereoBondWidth = m_DefaultStereoBondWidth * DefaultZoomFactor;
+		m_pData->ArrowPadding = m_DefaultArrowPadding * DefaultZoomFactor;
+		m_pData->SignPadding = m_DefaultSignPadding * DefaultZoomFactor;
 		m_pData->Group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (
 									gnome_canvas_root(GNOME_CANVAS(m_pWidget)),
 									gnome_canvas_group_ext_get_type (),
@@ -424,38 +426,9 @@ void gcpView::Print(GnomePrintContext *pc, gdouble width, gdouble height)
 void gcpView::Update(Object* pObject)
 {
 	std::list<GtkWidget*>::iterator i;
-	switch (pObject->GetType())
+	for (i = m_Widgets.begin(); i != m_Widgets.end(); i++)
 	{
-		case BondType:
-			for (i = m_Widgets.begin(); i != m_Widgets.end(); i++)
-			{
-				gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(*i), "data");
-				GnomeCanvasGroup* group = pData->Items[pObject];
-				if (group)
-				{
-					gtk_object_destroy(GTK_OBJECT(group));
-					pObject->Add(*i);//FIXME: replace with Update
-				}
-				else
-				{
-					pData->Items.erase(pObject);
-//					pObject->Update(*i);
-				}
-			}
-			break;
-		default:
-			if (pObject->HasChildren()) //FIXME: delete this when bonds update themselves
-			{
-				std::map<std::string, Object*>::iterator i;
-				Object* pObj;
-				for (pObj = pObject->GetFirstChild(i); pObj; pObj = pObject->GetNextChild(i)) Update(pObj);
-			}
-			else for (i = m_Widgets.begin(); i != m_Widgets.end(); i++)
-			{
-				pObject->Update(*i);
-			}
-			break;
-
+		pObject->Update(*i);
 	}
 }
 
@@ -778,5 +751,10 @@ void gcpView::OnSelectAll()
 	gcpTool* pActiveTool = pApp->GetTool("Select");
 	if (pActiveTool) pApp->ActivateTool("Select", true);
 	m_pData->SelectAll();
-	if (pActiveTool) pActiveTool->AddSelection(m_pData);
+	if (pActiveTool) {
+		pActiveTool->AddSelection(m_pData);
+		pApp->ActivateToolItem ("Merge", ((m_pData->SelectedObjects.size () == 2) &&
+			(m_pData->SelectedObjects.front()->GetType() == MoleculeType) &&
+			(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
+	}
 }
