@@ -27,10 +27,10 @@
 #ifndef GCHEMPAINT_WIDGET_DATA_H
 #define GCHEMPAINT_WIDGET_DATA_H
 
-#include <gnome.h>
+#include <libgnomecanvas/gnome-canvas.h>
 #include <map>
 #include <list>
-#include <chemistry/object.h>
+#include <gcu/object.h>
 
 using namespace gcu;
 
@@ -72,6 +72,8 @@ public:
 	double ArrowHeadB;
 	double ArrowHeadC;
 	double ArrowDist;
+	double ArrowPadding;
+	double SignPadding;
 	std::map<Object*, GnomeCanvasGroup*>Items;
 	std::list<Object*>SelectedObjects;
 	
@@ -81,6 +83,7 @@ public:
 	void UnselectAll();
 	void MoveSelectedItems(double dx, double dy);
 	void MoveSelection(double dx, double dy);
+	void RotateSelection(double dx, double dy, double angle);
 	bool ClearSelection() {SelectedObjects.clear();}
 	void Copy(GtkClipboard* clipboard);
 	void GetSelectionBounds(ArtDRect &rect);
@@ -88,6 +91,7 @@ public:
 	void SelectAll();
 	static xmlDocPtr GetXmlDoc(GtkClipboard* clipboard);
 	void ShowSelection(bool state);
+	void GetObjectBounds(Object* obj, ArtDRect *rect);
 
 private:
 	void MoveItems(Object *obj, double dx, double dy);
