@@ -26,7 +26,6 @@
 
 #include "gchempaint-config.h"
 #include <math.h>
-#include <libgnome/libgnome.h>
 #include "bondtool.h"
 #include "lib/settings.h"
 #include "lib/mendeleiev.h"
@@ -63,11 +62,6 @@ bool gcpBondTool::OnClicked()
 		{
 		case BondType:
 			pBond = (gcpBond*)m_pObject;
-			if ((pBond->GetType() == NormalBondType) && (pBond->GetOrder() < 3))
-			{//TODO: in the future, test maximum bond order for atoms to allow for quadruple bonds
-				if (!((gcpAtom*)pBond->GetAtom(0))->AcceptNewBonds()) return false;
-				if (!((gcpAtom*)pBond->GetAtom(1))->AcceptNewBonds()) return false;
-			}
 			m_pAtom = (gcpAtom*)pBond->GetAtom(0);
 			m_pAtom->GetCoords(&m_x0, &m_y0, NULL);
 			m_pAtom = (gcpAtom*)pBond->GetAtom(1);
@@ -256,6 +250,12 @@ void gcpBondTool::OnRelease()
 	if ((m_pObject) && (m_pObject->GetType() == BondType))
 	{
 		FinalizeBond();
+		gcpAtom* pAtom = (gcpAtom*) ((gcpBond*)m_pObject)->GetAtom(0);
+		pAtom->Update ();
+		m_pView->Update(pAtom);
+		pAtom = (gcpAtom*) ((gcpBond*)m_pObject)->GetAtom(1);
+		pAtom->Update ();
+		m_pView->Update(pAtom);
 		m_pOp->AddObject(m_pObject, 1);
 		pDoc->FinishOperation();
 		m_pOp = NULL;
@@ -298,9 +298,12 @@ void gcpBondTool::OnRelease()
 	}
 	if (m_pObject->GetType() == AtomType)
 	{
-		if (m_pAtom) pAtom = m_pAtom;
-		else
-		{
+		if (m_pAtom) {
+			if (m_pObject == m_pAtom) {
+				return;
+			}
+			pAtom = m_pAtom;
+		} else {
 			pAtom = new gcpAtom(m_pApp->GetCurZ(), m_x1 / m_dZoomFactor, m_y1 / m_dZoomFactor, 0);
 			pDoc->AddAtom(pAtom);
 		}
@@ -313,6 +316,12 @@ void gcpBondTool::OnRelease()
 			m_pObject = pBond;
 			m_bChanged = true;
 			FinalizeBond(); 
+			gcpAtom* pAtom = (gcpAtom*) ((gcpBond*)m_pObject)->GetAtom(0);
+			pAtom->Update ();
+			m_pView->Update(pAtom);
+			pAtom = (gcpAtom*) ((gcpBond*)m_pObject)->GetAtom(1);
+			pAtom->Update ();
+			m_pView->Update(pAtom);
 			m_pView->Update(pBond);
 			m_pOp->AddObject(pBond, 1);
 			pDoc->FinishOperation();
