@@ -33,14 +33,8 @@
 
 gcpSelectionPlugin plugin;
 
-static Object* CreateMolecule()
-{
-	return new gcpMolecule();
-}
-
 gcpSelectionPlugin::gcpSelectionPlugin(): gcpPlugin()
 {
-	Object::AddType("molecule", CreateMolecule, MoleculeType);
 }
 
 gcpSelectionPlugin::~gcpSelectionPlugin()
