@@ -41,12 +41,26 @@ public:
 	virtual bool OnClicked();
 	virtual void OnDrag();
 	virtual void OnRelease();
+	virtual void Activate();
 	virtual bool Deactivate();
+	virtual bool OnRightButtonClicked ();
 
 	void AddSelection(gcpWidgetData* data);
+	void OnFlip (bool horizontal);
+	void Rotate (bool rotate);
+	void Merge ();
+
+	void CreateGroup ();
 
 private:
 	std::list<gcpWidgetData*> SelectedWidgets;
+	bool m_bRotate;
+	double m_cx, m_cy;
+	double m_dAngle, m_dAngleInit;
+	gcpOperation* m_pOp;
+	GtkUIManager *m_UIManager;
+	TypeId m_Type;
+	list<int> m_uiIds;
 };
 
 #endif // GCHEMPAINT_SELECTION_TOOL_H
