@@ -30,6 +30,9 @@
 #include "lib/document.h"
 #include "lib/application.h"
 #include "lib/settings.h"
+#include <gdk/gdkkeysyms.h>
+#include <unistd.h>
+
 extern GtkTextTagTable* TextTagTable;
 
 static bool on_toggled(GtkWidget *btn, gcpApplication* App)
@@ -372,10 +375,12 @@ bool gcpTextTool::OnFont()
 			g_object_unref((GObject*)tag);
 		}
 		pango_font_description_free(pfd);
-		if (m_Active) gtk_text_buffer_apply_tag_by_name(buf, m_pApp->GetFontName(), &start, &end);
-		gcpText *text = (gcpText*)g_object_get_data(G_OBJECT(m_Active), "object");
-		text->OnChanged(buf);
-		if (gtk_text_iter_compare(&start, &end)) PushNode(text->SaveSelected());
+		if (m_Active) {
+			gtk_text_buffer_apply_tag_by_name(buf, m_pApp->GetFontName(), &start, &end);
+			gcpText *text = (gcpText*)g_object_get_data(G_OBJECT(m_Active), "object");
+			text->OnChanged(buf);
+			if (gtk_text_iter_compare(&start, &end)) PushNode(text->SaveSelected());
+		}
 	}
 	gtk_widget_destroy(dialog);
 	return true;
