@@ -25,30 +25,11 @@
  */
 
 #include "gchempaint-config.h"
-#include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
-#include <glade/glade.h>
-#include <list>
-#include <glib-object.h>
-#include <chemistry/chemistry.h>
-#include "lib/document.h"
-#include "lib/view.h"
-#include "lib/mendeleiev.h"
 #include "standaloneapp.h"
+#include <libgnomeui/libgnomeui.h>
 
 static GtkMenu *windowsmenu;
 
-/*
-static char* opt_filename = NULL;
-static bool bonobo_server_flag = false;
-struct poptOption options[] =
-{
-    {"bonobo-server", '\0', POPT_ARG_NONE, &bonobo_server_flag, 0, N_("Allow GChemPaint to act as a Bonobo server."), NULL},
-	{NULL, '\0', 0, NULL, 0, NULL, NULL}
-};
-*/
-
 #warning "the following lines should be removed for stable releases"
 #undef PACKAGE
 #define PACKAGE "gchempaint-unstable" 
@@ -63,11 +44,10 @@ int main(int argc, char *argv[])
 	textdomain(GETTEXT_PACKAGE);
 
 	GnomeProgram* prog = gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv, 
- //                  GNOME_PARAM_POPT_TABLE, options,
+//                   GNOME_PARAM_POPT_TABLE, NULL,
                    GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	g_object_get(G_OBJECT(prog), GNOME_PARAM_POPT_CONTEXT, &pctx, NULL);
 
-
 /*	if (bonobo_server_flag)
 	{
 		BONOBO_FACTORY_INIT ("gchempaint", VERSION, &argc, argv);		
@@ -81,7 +61,7 @@ int main(int argc, char *argv[])
 		if (args)
 			while(args[i])
 			{
-				App->FileProcess(args[i++], 0, false);
+				App->FileProcess(args[i++], 0, false, NULL);
 			}	
 		else App->OnFileNew();
 
