@@ -89,12 +89,11 @@ void on_print(GtkWidget* widget, gcpStandaloneApp* App)
 
 void on_show_elements(GtkWidget* widget, gcpStandaloneApp* App)
 {
-/*	if (MendeleievDlg) MendeleievDlg->Destroy();
+	gcpDialog *dlg = App->GetDialog ("Mendeleiev");
+	if (dlg)
+		dlg->Destroy ();
 	else
-	{
-		MendeleievDlg = new gcpMendeleievDlg(CurElt);
-		MendeleievDlg->SetCallBack(&on_change_element);
-	}*/
+		App->OnShowElements ();
 }
 
 static void on_cut_selection(GtkWidget* widget, gcpStandaloneApp* App)
@@ -149,6 +148,16 @@ static void on_prefs(GtkWidget* widget, gcpStandaloneApp* App)
 	gtk_widget_destroy(GTK_WIDGET(dlg));*/
 }
 
+static void on_close_all(GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->OnCloseAll ();
+}
+
+static void on_save_all(GtkWidget* widget, gcpStandaloneApp* App)
+{
+	App->OnSaveAll ();
+}
+
 static bool on_focus(GtkWidget *widget, GdkEventFocus *event, gcpStandaloneApp* App)
 {
 	gtk_clipboard_request_contents(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc)on_receive_targets, App);
@@ -161,121 +170,293 @@ static bool on_tool_clicked(GtkWidget *widget, gcpStandaloneApp* App)
 	return false;
 }
 
+static void on_help (GtkWidget *widget, gcpStandaloneApp* App)
+{
+	App->OnHelp ();
+}
+
+static void on_web (GtkWidget *widget, gcpStandaloneApp* App)
+{
+	App->OnWeb ();
+}
+
+static void on_mail (GtkWidget *widget, gcpStandaloneApp* App)
+{
+	App->OnMail ();
+}
+
+static void on_bug (GtkWidget *widget, gcpStandaloneApp* App)
+{
+	App->OnBug ();
+}
+
+static void on_show_menu_tip (GtkWidget *proxy, gcpStandaloneApp* App)
+{
+	GtkAction *action = (GtkAction*) g_object_get_data (G_OBJECT (proxy), "action");
+	char *tip;
+	g_object_get (action, "tooltip", &tip, NULL);
+	if (tip != NULL){
+		App->SetStatusText (tip);
+		g_free (tip);
+	}
+}
+
+static void on_clear_menu_tip (gcpStandaloneApp* App)
+{
+		App->ClearStatus ();
+}
+
+static void on_connect_proxy (GtkUIManager *ui, GtkAction *action, GtkWidget *proxy, gcpStandaloneApp* App)
+{
+	/* connect whether there is a tip or not it may change later */
+	if (GTK_IS_MENU_ITEM (proxy)) {
+		g_object_set_data (G_OBJECT (proxy), "action", action);
+		g_object_connect (proxy,
+			"signal::select",  G_CALLBACK (on_show_menu_tip), App,
+			"swapped_signal::deselect", G_CALLBACK (on_clear_menu_tip), App,
+			NULL);
+	}
+}
+
+static void on_disconnect_proxy (GtkUIManager *ui, GtkAction *action, GtkWidget *proxy, gcpStandaloneApp* App)
+{
+	if (GTK_IS_MENU_ITEM (proxy)) {
+		g_object_set_data (G_OBJECT (proxy), "action", NULL);
+		g_object_disconnect (proxy,
+			"any_signal::select",  G_CALLBACK (on_show_menu_tip), App,
+			"any_signal::deselect", G_CALLBACK (on_clear_menu_tip), App,
+			NULL);
+	}
+}
+
+
 /*********
  * Menus *
  *********/
-	
-static GnomeUIInfo file_menu [] = {
-	GNOMEUIINFO_MENU_NEW_ITEM(N_("_New File"), N_("Create a new file"), on_file_new, NULL),
-	GNOMEUIINFO_MENU_OPEN_ITEM(on_file_open, NULL),
-	GNOMEUIINFO_MENU_SAVE_ITEM(on_file_save, NULL),
-	GNOMEUIINFO_MENU_SAVE_AS_ITEM(on_file_save_as, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_PRINT_ITEM(on_print, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_PROPERTIES_ITEM(on_properties, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_CLOSE_ITEM(on_file_close, NULL),
-	GNOMEUIINFO_MENU_EXIT_ITEM(on_quit, NULL),
-	GNOMEUIINFO_END
+static GtkActionEntry entries[] = {
+  { "FileMenu", NULL, N_("_File") },
+	  { "New", GTK_STOCK_NEW, N_("_New File"), NULL,
+		  N_("Create a new file"), G_CALLBACK (on_file_new) },
+	  { "Open", GTK_STOCK_OPEN, N_("_Open"), "<control>O",
+		  N_("Open a file"), G_CALLBACK (on_file_open) },
+	  { "Save", GTK_STOCK_SAVE, N_("_Save"), "<control>S",
+		  N_("Save the current file"), G_CALLBACK (on_file_save) },
+	  { "SaveAs", GTK_STOCK_SAVE, N_("Save _As"), "<shift><control>S",
+		  N_("Save the current file with a different name "), G_CALLBACK (on_file_save_as) },
+	  { "Print", GTK_STOCK_PRINT, N_("_Print"), "<control>P",
+		  N_("Print the current file"), G_CALLBACK (on_print) },
+	  { "Properties", GTK_STOCK_PROPERTIES, N_("Prope_rties"), NULL,
+		  N_("Modify the file's properties"), G_CALLBACK (on_properties) },
+	  { "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
+		  N_("Close the current file"), G_CALLBACK (on_file_close) },
+	  { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
+		  N_("Quit GChemPaint"), G_CALLBACK (on_quit) },
+  { "EditMenu", NULL, N_("_Edit") },
+	  { "Undo", GTK_STOCK_UNDO, N_("_Undo"), "<control>Z",
+		  N_("Undo the lase action"), G_CALLBACK (on_undo) },
+	  { "Redo", GTK_STOCK_REDO, N_("_Redo"), "<shift><control>Z",
+		  N_("Redo the undone action"), G_CALLBACK (on_redo) },
+	  { "Cut", GTK_STOCK_CUT, N_("Cu_t"), "<control>X",
+		  N_("Cut the selection"), G_CALLBACK (on_cut_selection) },
+	  { "Copy", GTK_STOCK_COPY, N_("_Copy"), "<control>C",
+		  N_("Copy the selection"), G_CALLBACK (on_copy_selection) },
+	  { "Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V",
+		  N_("Paste the clipborad"), G_CALLBACK (on_paste_selection) },
+	  { "Delete", GTK_STOCK_CLEAR, N_("C_lear"), NULL,
+		  N_("Clear the selection"), G_CALLBACK (on_delete_selection) },
+	  { "SelectAll", NULL, N_("Select _All"), "<control>A",
+		  N_("Select everything"), G_CALLBACK (on_select_all) },
+  { "ViewMenu", NULL, N_("_View") },
+	  { "ToolbarMenu", NULL, N_("_Toolbars") },
+  { "WindowsMenu", NULL, N_("_Windows") },
+	  { "SaveAll", GTK_STOCK_SAVE, N_("_Save All"), "<shift><control>L",
+		  N_("Save all open files"), G_CALLBACK (on_save_all) },
+	  { "CloseAll", GTK_STOCK_CLOSE, N_("_Close All"), "<control>W",
+		  N_("Close all open files"), G_CALLBACK (on_close_all) },
+  { "HelpMenu", NULL, N_("_Help") },
+	  { "Help", GTK_STOCK_HELP, N_("_Contents"), "F1",
+		  N_("View help for GChemPaint"), G_CALLBACK (on_help) },
+	  { "Web", NULL, N_("GChemPaint on the _web"), NULL,
+		  N_("Browse GChemPaint's web site"), G_CALLBACK (on_web) },
+	  { "Mail", NULL, N_("_Ask a question"), NULL,
+		  N_("Ask a question about GChemPaint"), G_CALLBACK (on_mail) },
+	  { "Bug", NULL, N_("Report _Bugs"), NULL,
+		  N_("Submit a bug report for GChemPaint"), G_CALLBACK (on_bug) },
+	  { "About", NULL, N_("_About"), NULL,
+		  N_("About GChemPaint"), G_CALLBACK (on_about) }
 };
- 
-static GnomeUIInfo edit_menu [] = {
-	GNOMEUIINFO_MENU_UNDO_ITEM(on_undo, NULL),
-	GNOMEUIINFO_MENU_REDO_ITEM(on_redo, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_CUT_ITEM(on_cut_selection, NULL),
-	GNOMEUIINFO_MENU_COPY_ITEM(on_copy_selection, NULL),
-	GNOMEUIINFO_MENU_PASTE_ITEM(on_paste_selection, NULL),
-	GNOMEUIINFO_MENU_CLEAR_ITEM(on_delete_selection, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_SELECT_ALL_ITEM(on_select_all, NULL),
-	GNOMEUIINFO_END
- };
 
-#define TOOLS		0
-#define MENDELEIEV	1
+/* Toggle items */
+static GtkToggleActionEntry toggle_entries[] = {
+	  { "Mendeleiev", NULL, N_("_Periodic table"), NULL,
+		  N_("Show the periodic table of the elements"), G_CALLBACK (on_show_elements), false }
+};
+
+static const char *ui_description =
+"<ui>"
+"  <menubar name='MainMenu'>"
+"    <menu action='FileMenu'>"
+"      <menuitem action='New'/>"
+"      <menuitem action='Open'/>"
+"      <menuitem action='Save'/>"
+"      <menuitem action='SaveAs'/>"
+"      <separator name='file-sep1'/>"
+"      <menuitem action='Print'/>"
+"      <separator name='file-sep2'/>"
+"      <menuitem action='Properties'/>"
+"      <separator name='file-sep3'/>"
+"      <menuitem action='Close'/>"
+"      <menuitem action='Quit'/>"
+"    </menu>"
+"    <menu action='EditMenu'>"
+"      <menuitem action='Undo'/>"
+"      <menuitem action='Redo'/>"
+"      <separator name='edit-sep1'/>"
+"      <menuitem action='Cut'/>"
+"      <menuitem action='Copy'/>"
+"      <menuitem action='Paste'/>"
+"      <menuitem action='Delete'/>"
+"      <separator name='edit-sep2'/>"
+"      <menuitem action='SelectAll'/>"
+"    </menu>"
+"    <menu action='ViewMenu'>"
+"      <menu action='ToolbarMenu'>"
+"      </menu>"
+"      <menuitem action='Mendeleiev'/>"
+"    </menu>"
+"    <menu action='WindowsMenu'>"
+"      <menuitem action='SaveAll'/>"
+"      <menuitem action='CloseAll'/>"
+"      <separator name='windows-sep1'/>"
+"      <placeholder name='windows'/>"
+"    </menu>"
+"    <menu action='HelpMenu'>"
+"      <menuitem action='Help'/>"
+"      <placeholder name='web'/>"
+"      <placeholder name='mail'/>"
+"      <placeholder name='bug'/>"
+"      <menuitem action='About'/>"
+"    </menu>"
+"  </menubar>"
+"  <toolbar name='MainToolbar'>"
+"    <toolitem action='New'/>"
+"    <toolitem action='Open'/>"
+"    <toolitem action='Save'/>"
+"    <toolitem action='Print'/>"
+"    <toolitem action='Close'/>"
+"    <toolitem action='Quit'/>"
+"  </toolbar>"
+"</ui>";
+
+static const char *ui_mail_description =
+"<ui>"
+"  <menubar name='MainMenu'>"
+"    <menu action='HelpMenu'>"
+"      <placeholder name='mail'>"
+"        <menuitem action='Mail'/>"
+"      </placeholder>"
+"    </menu>"
+"  </menubar>"
+"</ui>";
+
+static const char *ui_web_description =
+"<ui>"
+"  <menubar name='MainMenu'>"
+"    <menu action='HelpMenu'>"
+"      <placeholder name='web'>"
+"        <menuitem action='Web'/>"
+"      </placeholder>"
+"      <placeholder name='bug'>"
+"        <menuitem action='Bug'/>"
+"      </placeholder>"
+"    </menu>"
+"  </menubar>"
+"</ui>";
 
 #warning "the following lines should be removed for stable releases"
 #undef PACKAGE
 #define PACKAGE "gchempaint-unstable" 
 
-static GnomeUIInfo tools_menu [] = {
-//	GNOMEUIINFO_TOGGLEITEM(N_("_Main"), N_("Show main tool bar"), on_show_tools, GCHEMPAINT_MAIN_TOOLBAR),
-//	GNOMEUIINFO_TOGGLEITEM(N_("_Text"), N_("Show text tool bar"), on_show_tools, GCHEMPAINT_TEXT_TOOLBAR),
-	GNOMEUIINFO_END
-};
-
-static GnomeUIInfo view_menu [] = {
-	GNOMEUIINFO_SUBTREE(N_("_Toolbars"), tools_menu),
-	GNOMEUIINFO_TOGGLEITEM(N_("_Periodic table"), N_("Show the periodic table of the elements"), on_show_elements, NULL),
-	GNOMEUIINFO_END
- };
- 
-static GnomeUIInfo windows_menu [] = {
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_END
- };
- 
-static GnomeUIInfo help_menu [] = {
-	GNOMEUIINFO_HELP(PACKAGE),
-	GNOMEUIINFO_MENU_ABOUT_ITEM(on_about, NULL),
-	GNOMEUIINFO_END
- };
-
-static GnomeUIInfo main_menu [] = {
-	GNOMEUIINFO_MENU_FILE_TREE(&file_menu),
-	GNOMEUIINFO_MENU_EDIT_TREE(&edit_menu),
-	GNOMEUIINFO_MENU_VIEW_TREE(&view_menu),
-	GNOMEUIINFO_MENU_WINDOWS_TREE(&windows_menu),
-	GNOMEUIINFO_MENU_HELP_TREE(&help_menu),
-	GNOMEUIINFO_END
-};
-
-static GnomeUIInfo tool_bar [] = {
-	GNOMEUIINFO_ITEM_STOCK(N_("New"), N_("Create a new file"), on_file_new, GTK_STOCK_NEW),
-	GNOMEUIINFO_ITEM_STOCK(N_("Open"), N_("Open a file"), on_file_open, GTK_STOCK_OPEN),
-	GNOMEUIINFO_ITEM_STOCK(N_("Save"), N_("Save active file"), on_file_save, GTK_STOCK_SAVE),
-	GNOMEUIINFO_ITEM_STOCK(N_("Print"), N_("Print active file"), on_print, GTK_STOCK_PRINT),
-	GNOMEUIINFO_ITEM_STOCK(N_("Close"), N_("Close active file"), on_file_close, GTK_STOCK_CLOSE),
-	GNOMEUIINFO_ITEM_STOCK(N_("Quit"), N_("Quit GChemPaint"), on_quit, GTK_STOCK_QUIT),
-	GNOMEUIINFO_END
-};
-
 gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 {
+	GtkWidget *vbox;
+	GtkWidget *bar;
+	GtkWidget *item;
+	GtkActionGroup *action_group;
+	GtkAccelGroup *accel_group;
+	GError *error;
+
 	m_NumWindow = 1;
-	m_App = GNOME_APP(gnome_app_new(_("GChemPaint"), _("GChemPaint")));
-	gnome_app_create_menus_with_data(m_App, main_menu, this);
-	GtkToolbar* Toolbar = GTK_TOOLBAR(gtk_toolbar_new());
-	gnome_app_fill_toolbar_with_data(Toolbar, tool_bar, NULL, this);
-	gtk_toolbar_set_style(Toolbar, GTK_TOOLBAR_ICONS);
-	gnome_app_add_toolbar(m_App, GTK_TOOLBAR(Toolbar), _("Main"), BONOBO_DOCK_ITEM_BEH_NORMAL, BONOBO_DOCK_TOP, 1, 0, 0);
-	SetMenu("Undo", edit_menu[0].widget);
-	SetMenu("Redo", edit_menu[1].widget);
-	SetMenu("Copy", edit_menu[3].widget);
-	SetMenu("Cut", edit_menu[4].widget);
-	SetMenu("Paste", edit_menu[5].widget);
-	SetMenu("Erase", edit_menu[6].widget);
-	SetMenu("Windows", windows_menu[0].widget->parent);
-	m_Bar = gnome_appbar_new(true, true, GNOME_PREFERENCES_ALWAYS);
-	gnome_appbar_set_default(GNOME_APPBAR(m_Bar), _("Ready."));
-	gnome_app_set_statusbar(m_App, m_Bar);
-	gnome_app_install_appbar_menu_hints(GNOME_APPBAR(m_Bar), main_menu);
-	g_signal_connect(G_OBJECT(m_App), "destroy", G_CALLBACK(on_quit), this);
-	g_signal_connect(G_OBJECT(m_App), "focus_in_event", G_CALLBACK(on_focus), this);
+	m_NumDoc = 0;
+	m_MessageId = 0;
+	m_Window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
+	gtk_window_set_title (m_Window, _("GChemPaint"));
+	gtk_window_set_icon_from_file (m_Window, PIXMAPSDIR"/gchempaint.png", NULL);
+	g_signal_connect(G_OBJECT(m_Window), "destroy", G_CALLBACK(on_quit), this);
+	g_signal_connect(G_OBJECT(m_Window), "focus_in_event", G_CALLBACK(on_focus), this);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (m_Window), vbox);
+	action_group = gtk_action_group_new ("MenuActions");
+	gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), this);
+	gtk_action_group_add_toggle_actions (action_group, toggle_entries, G_N_ELEMENTS (toggle_entries), this);
+
+	m_UIManager = gtk_ui_manager_new ();
+	g_object_connect (m_UIManager,
+		"signal::connect_proxy",    G_CALLBACK (on_connect_proxy), this,
+		"signal::disconnect_proxy", G_CALLBACK (on_disconnect_proxy), this,
+		NULL);
+	gtk_ui_manager_insert_action_group (m_UIManager, action_group, 0);
+	if (MailAgent[0])
+
+	accel_group = gtk_ui_manager_get_accel_group (m_UIManager);
+	gtk_window_add_accel_group (GTK_WINDOW (m_Window), accel_group);
+	
+	if (MailAgent[0])
+	error = NULL;
+	if (!gtk_ui_manager_add_ui_from_string (m_UIManager, ui_description, -1, &error))
+	  {
+		g_message ("building menus failed: %s", error->message);
+		g_error_free (error);
+		exit (EXIT_FAILURE);
+	  }
+	if (WebBrowser[0])
+		gtk_ui_manager_add_ui_from_string (m_UIManager, ui_web_description, -1, &error);
+	if (MailAgent[0])
+		gtk_ui_manager_add_ui_from_string (m_UIManager, ui_mail_description, -1, &error);
+
+	bar = gtk_ui_manager_get_widget (m_UIManager, "/MainMenu");
+	gtk_box_pack_start (GTK_BOX (vbox), bar, false, false, 0);
+	m_Dock = bonobo_dock_new ();
+	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (m_Dock), true, true, 0);
+	bar = gtk_ui_manager_get_widget (m_UIManager, "/MainToolbar");
+	gtk_toolbar_set_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_ICONS);
+	gtk_toolbar_set_show_arrow(GTK_TOOLBAR(bar), false);
+	gtk_toolbar_set_tooltips(GTK_TOOLBAR(bar), true);
+	item = bonobo_dock_item_new (_("Main"), BONOBO_DOCK_ITEM_BEH_NORMAL);
+	gtk_container_add (GTK_CONTAINER (item), bar);
+	bonobo_dock_add_item (BONOBO_DOCK (m_Dock),BONOBO_DOCK_ITEM (item), BONOBO_DOCK_TOP, 0, 0, 0, true);
+
 	m_Book = GTK_NOTEBOOK(gtk_notebook_new());
 	gtk_notebook_set_tab_pos(m_Book, GTK_POS_TOP);
-	gnome_app_set_contents(m_App, GTK_WIDGET(m_Book));
+	bonobo_dock_set_client_area (BONOBO_DOCK (m_Dock), GTK_WIDGET (m_Book));
+
+	m_Bar = gtk_statusbar_new ();
+	m_statusId = gtk_statusbar_get_context_id (GTK_STATUSBAR (m_Bar), "status");
+	gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, _("Ready"));
+	gtk_box_pack_start (GTK_BOX (vbox), m_Bar, false, false, 0);
+	gtk_widget_show_all (GTK_WIDGET (m_Window));
+	SetMenu("Undo", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Undo"));
+	SetMenu("Redo", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Redo"));
+	SetMenu("Copy", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Copy"));
+	SetMenu("Cut", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Cut"));
+	SetMenu("Paste", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Paste"));
+	SetMenu("Erase", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Delete"));
 	ActivateMenu("Undo", false);
 	ActivateMenu("Redo", false);
 	ActivateMenu("Copy", false);
 	ActivateMenu("Cut", false);
 	ActivateMenu("Paste", false);
 	ActivateMenu("Erase", false);
-		
-	g_signal_connect(GTK_OBJECT(m_App), "key_press_event", (GCallback)on_key_press, this);
-	g_signal_connect(GTK_OBJECT(m_App), "key_release_event", (GCallback)on_key_release, this);
 
 	Callbacks["tools"] = (GCallback) on_tool_clicked;
 
@@ -283,9 +464,13 @@ gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 		BonoboUINode* Child = bonobo_ui_node_children (m_pNode);
 		AppendUI (Child);
 	}
-	gtk_widget_show_all(GTK_WIDGET(m_App));
+		
+	g_signal_connect(GTK_OBJECT(m_Window), "key_press_event", (GCallback)on_key_press, this);
+	g_signal_connect(GTK_OBJECT(m_Window), "key_release_event", (GCallback)on_key_release, this);
 
-	SetMenu("Mendeleiev", view_menu[MENDELEIEV].widget);
+	Callbacks["tools"] = (GCallback) on_tool_clicked;
+
+	SetMenu("Mendeleiev", gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/ViewMenu/Mendeleiev"));
 	gcpMendeleievDlg::SetElement (this, m_CurZ);
 }
 
@@ -336,13 +521,24 @@ void gcpStandaloneApp::OnFileNew()
 	if (w)
 	{
 		GtkWidget *label = gtk_label_new(tmp);
-		GtkWidget* item = gtk_menu_item_new_with_label(tmp);
+		g_snprintf(tmp, sizeof(tmp), "win%d", m_NumDoc++);
+		GtkAction *action = gtk_action_new (tmp, gtk_label_get_text (GTK_LABEL (label)), NULL, NULL);
+		GtkActionGroup *group = gtk_action_group_new (tmp);
+		GError* error;
+		gtk_action_group_add_action (group, action);
+		gtk_ui_manager_insert_action_group (m_UIManager, group, 0);
+		gchar* buf = g_strconcat ("<ui><menubar name='MainMenu'><menu action='WindowsMenu'><menuitem action='",
+		tmp, "'/></menu></menubar></ui>", NULL);
+		error = NULL;
+		int id = gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL);
+		g_free (buf);
+		buf = g_strconcat ("/MainMenu/WindowsMenu/", tmp, NULL);
+		GtkWidget* item = gtk_ui_manager_get_widget (m_UIManager, buf);
+		g_free (buf);
 		g_signal_connect(G_OBJECT(item), "activate", (GtkSignalFunc)on_select, w); 
 		g_signal_connect(G_OBJECT(w), "expose_event", (GtkSignalFunc)on_page_select, NULL); 
-		g_object_set_data(G_OBJECT(w), "menu", item);
+		g_object_set_data(G_OBJECT(w), "menu-id", (void*) id);
 		g_object_set_data(G_OBJECT(w), "label", label);
-		gtk_widget_show(item);
-		gtk_menu_shell_append(GTK_MENU_SHELL(GetMenu("Windows")), item);
 		GtkScrolledWindow* scroll = (GtkScrolledWindow*)gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
@@ -372,14 +568,26 @@ void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
 	if (w)
 	{
 		const char* tmp = pDoc->GetLabel();
+		char winname[32];
 		GtkWidget *label = gtk_label_new(tmp);
-		GtkWidget* item = gtk_menu_item_new_with_label(tmp);
+		g_snprintf(winname, sizeof(winname), "win%d", m_NumDoc++);
+		GtkAction *action = gtk_action_new (winname, tmp, NULL, NULL);
+		GtkActionGroup *group = gtk_action_group_new (winname);
+		GError* error;
+		gtk_action_group_add_action (group, action);
+		gtk_ui_manager_insert_action_group (m_UIManager, group, 0);
+		gchar* buf = g_strconcat ("<ui><menubar name='MainMenu'><menu action='WindowsMenu'><menuitem action='",
+		winname, "'/></menu></menubar></ui>", NULL);
+		error = NULL;
+		int id = gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL);
+		g_free (buf);
+		buf = g_strconcat ("/MainMenu/WindowsMenu/", winname, NULL);
+		GtkWidget* item = gtk_ui_manager_get_widget (m_UIManager, buf);
+		g_free (buf);
 		g_signal_connect(G_OBJECT(item), "activate", (GtkSignalFunc)on_select, w); 
 		g_signal_connect(G_OBJECT(w), "expose_event", (GtkSignalFunc)on_page_select, NULL); 
-		g_object_set_data(G_OBJECT(w), "menu", item);
+		g_object_set_data(G_OBJECT(w), "menu-id", (void*) id);
 		g_object_set_data(G_OBJECT(w), "label", label);
-		gtk_widget_show(item);
-		gtk_menu_shell_append(GTK_MENU_SHELL(GetMenu("Windows")), item);
 		GtkScrolledWindow* scroll = (GtkScrolledWindow*)gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
@@ -394,6 +602,8 @@ void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
 
 void gcpStandaloneApp::RemoveDocument(gcpDocument* pDoc)
 {
+	GtkWidget *w = pDoc->GetWidget ();
+	gtk_ui_manager_remove_ui (m_UIManager, (int) g_object_get_data (G_OBJECT (w), "menu-id"));
 	m_Docs.remove (pDoc);
 }
 
@@ -430,36 +640,44 @@ void gcpStandaloneApp::ActivateToolItem(const string& itemname, bool activate)
 
 void gcpStandaloneApp::ClearStatus()
 {
-	gnome_appbar_clear_stack(GNOME_APPBAR(m_Bar));
+	if (m_MessageId) {
+		gtk_statusbar_pop (GTK_STATUSBAR (m_Bar), m_statusId);
+		m_MessageId = 0;
+	}
 }
 
 void gcpStandaloneApp::SetStatusText(const char* text)
 {
-	gnome_appbar_set_status(GNOME_APPBAR(m_Bar), text);
+	if (m_MessageId) gtk_statusbar_pop (GTK_STATUSBAR (m_Bar), m_statusId);
+	m_MessageId = gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, text);
 }
 
 GtkWindow* gcpStandaloneApp::GetWindow()
 {
-	return GTK_WINDOW (m_App);
+	return m_Window;
 }
 
 void gcpStandaloneApp::AppendUI (BonoboUINode* Node)
 {
+	map<unsigned, GtkWidget*>toolbars;
 	while (Node)
 	{
 		if (!strcmp(bonobo_ui_node_get_name(Node), "dockitem"))
 		{
-			AddToolbar (Node);
+			AddToolbar (Node, &toolbars);
 		}
 		Node = bonobo_ui_node_next (Node);
 	}
+	map<unsigned, GtkWidget*>::iterator i, end = toolbars.end();
+	for (i = toolbars.begin(); i != end; i++)
+		bonobo_dock_add_item (BONOBO_DOCK (m_Dock), BONOBO_DOCK_ITEM ((*i).second), BONOBO_DOCK_TOP, (*i).first >> 8, (*i).first & 0xff, 0, 0);
 	GtkToggleToolButton* button = (GtkToggleToolButton*) ToolItems["Select"];
 	if (button && !gtk_toggle_tool_button_get_active (button))
 		gtk_toggle_tool_button_set_active (button, true);
 	InitTools();
 }
 
-void gcpStandaloneApp::AddToolbar (BonoboUINode* Node)
+void gcpStandaloneApp::AddToolbar (BonoboUINode* Node, map<unsigned, GtkWidget*> *toolbars)
 {
 	if (!bonobo_ui_node_has_attr (Node, "name")) return;
 	if (!bonobo_ui_node_has_attr (Node, "position")) return;
@@ -479,7 +697,7 @@ void gcpStandaloneApp::AddToolbar (BonoboUINode* Node)
 		bonobo_ui_node_free_string (name);
 		name = NULL;
 	}
-	name = bonobo_ui_node_get_attr (Node, "band_num");
+	name = bonobo_ui_node_get_attr (Node, "position");
 	int rank = (name)? atoi (name): 0;
 	if (name)
 	{
@@ -490,10 +708,15 @@ void gcpStandaloneApp::AddToolbar (BonoboUINode* Node)
 	GtkToolbar* toolbar = GTK_TOOLBAR (gtk_toolbar_new());
 	Toolbars[name] = (GtkWidget*)toolbar;
 	gtk_toolbar_set_show_arrow(toolbar, false);
-	gtk_widget_show_all(GTK_WIDGET(toolbar));
 	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
 	gtk_toolbar_set_tooltips(GTK_TOOLBAR(toolbar), true);
-	gnome_app_add_toolbar(m_App, toolbar, name, BONOBO_DOCK_ITEM_BEH_NORMAL, BONOBO_DOCK_TOP, row, rank, in_new_band);
+	GtkWidget *item = bonobo_dock_item_new (name, BONOBO_DOCK_ITEM_BEH_NORMAL);
+	gtk_container_add (GTK_CONTAINER (item), GTK_WIDGET (toolbar));
+	gtk_widget_show_all(item);
+	if (toolbars && !in_new_band)
+		(*toolbars)[(row << 8) + rank] = item;
+	else
+		bonobo_dock_add_item (BONOBO_DOCK (m_Dock), BONOBO_DOCK_ITEM (item), BONOBO_DOCK_TOP, row, rank, 0, in_new_band);
 	if (name)
 	{
 		bonobo_ui_node_free_string (name);
@@ -530,9 +753,9 @@ void gcpStandaloneApp::AddToolbar (BonoboUINode* Node)
 						bonobo_ui_node_free_string (build);
 					}
 				}
+				bonobo_ui_node_free_string (type);
 			}
 			else button = gtk_tool_button_new (NULL, NULL);
-			bonobo_ui_node_free_string (type);
 			if (GTK_IS_TOOL_BUTTON (button)) {
 				if (bonobo_ui_node_has_attr (Child, "pixtype")) {
 					const char* filename = NULL;
@@ -568,13 +791,13 @@ void gcpStandaloneApp::AddToolbar (BonoboUINode* Node)
 				name = bonobo_ui_node_get_attr (Child, "verb");
 				if (Callbacks[name])
 					g_signal_connect (button, "clicked", Callbacks[name], this);
-//				bonobo_ui_node_free_string (name);
+				bonobo_ui_node_free_string (name);
 			}
 			if (bonobo_ui_node_has_attr (Child, "name")) {
 				name = bonobo_ui_node_get_attr (Child, "name");
 				gtk_widget_set_name(GTK_WIDGET(button), name);
 				SetToolItem (name, (GtkWidget*) button);
-//				bonobo_ui_node_free_string (name);
+				bonobo_ui_node_free_string (name);
 			}
 			pos = -1;
 			if (bonobo_ui_node_has_attr (Child, "pos")) {
@@ -582,12 +805,12 @@ void gcpStandaloneApp::AddToolbar (BonoboUINode* Node)
 				if (!strcmp (name, "top")) pos = 0;
 				else if (!strcmp (name, "bottom"));
 				else pos = atoi (name);
-//				bonobo_ui_node_free_string (name);
+				bonobo_ui_node_free_string (name);
 			}
 			if (bonobo_ui_node_has_attr (Child, "tip")) {
 				name = bonobo_ui_node_get_attr (Child, "tip");
 				gtk_tool_item_set_tooltip (button, toolbar->tooltips, name, NULL);
-//				bonobo_ui_node_free_string (name);
+				bonobo_ui_node_free_string (name);
 			}
 			if (pos > (rank = gtk_toolbar_get_n_items (toolbar)))
 				pos = rank;
@@ -672,7 +895,8 @@ void gcpStandaloneApp::SetActive (gcpDocument* pDoc, GtkWidget* w)
 void gcpStandaloneApp::OnFileClose ()
 {
 	gcpView* pView = m_pActiveDoc->GetView ();
-	pView->PrepareUnselect ();
+	pView->PrepareUnselect (); 
+	m_pActiveDoc = NULL;
 	gint i;
 	i = gtk_notebook_get_current_page (m_Book);
 	gtk_notebook_remove_page (m_Book, i);
@@ -748,3 +972,35 @@ void gcpStandaloneApp::OnDeleteSelection()
 	gcpView* v = (gcpView*) g_object_get_data(G_OBJECT(w), "view");
 	if (v && v->GetDoc()->GetEditable()) v->OnDeleteSelection(w);
 }
+
+void gcpStandaloneApp::OnCloseAll ()
+{
+	while (m_Docs.size ()) {
+		OnFileClose ();
+		while (gtk_events_pending())
+			gtk_main_iteration();
+	}
+}
+
+void gcpStandaloneApp::OnSaveAll ()
+{
+	gcpDocument* pDoc = m_pActiveDoc;
+	list<gcpDocument*>::iterator i, end = m_Docs.end ();
+	for (i = m_Docs.begin (); i != end; i++) {
+		if (!(*i)->IsDirty ())
+			continue;
+		on_select (NULL, (*i)->GetWidget ());
+		OnSave ();
+	}
+	if (pDoc)
+		on_select (NULL, pDoc->GetWidget ());
+}
+
+void gcpStandaloneApp::ToggleMenu(const string& menuname, bool active)
+{
+	GtkWidget* w = Menus[menuname];
+	if (!GTK_IS_CHECK_MENU_ITEM (w))
+		return;
+	if (gtk_check_menu_item_get_active ((GtkCheckMenuItem*) w) != active)
+		((GtkCheckMenuItem*) w)->active = active;
+}
