@@ -1,6 +1,4 @@
 #include "gchempaint-config.h"
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
 #include <libbonobo.h>
 #include <libbonoboui.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -64,9 +62,7 @@ main (int argc, char** argv)
 	GnomeProgram* prog;
 	
 	CORBA_exception_init (&ev);
-	prog = gnome_program_init ("bonobo-control-test", "0.0", LIBGNOMEUI_MODULE, argc, argv, 
-                   GNOME_PARAM_POPT_TABLE, NULL, 
-                   GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
+	gtk_init (&argc, &argv);
 
 	/*
 	 * initialize CORBA, OAF  and bonobo
